#!/usr/bin/perl -w

# genbf.pl - generate a big, random, uncompressable, undeduplicatable
# file.
#
# (C) 2014 Preston de Guise
# You may modify this script as you wish, but you may not charge for it.

use strict;
use Getopt::Std;
use File::Basename;

my $dataChunkSize = 10485760;	# 10MB
my $randomChars;
my $filename = "";
my $sizeMB = 10;
my $size = $sizeMB * 1024 * 1024;
my $self = basename($0);
my $subChunkDivider = 50;
my $segmentDivider = 100;
my %opts;

##############################################################################
# Subroutines
##############################################################################

# usage() shows usage information and then dies.
sub usage {
	print "
Usage: $self [-h] [-c size] [-C div] [-S div] -s size -f filename

Generates a big random file.

-h		Print this help and exit.
-c size		Chunk size, in MB (default is 10MB, suitable for files in
		the GB range.)
-C div		Divider to create sub-chunks. Default is $subChunkDivider.
-S div		Divider to create segments. Default is $segmentDivider.
-s size		File size in MB, default is 10MB.
-f filename	File to write to.

";

	if (@_+0 != 0) {
		my @messages = @_;
		foreach my $message (@messages) {
			my $tmp = $message;
			chomp $tmp;
			print "$tmp\n";
		}
	}
	die "\n";
}

# randfile($size,$filename) generates a random file from $randomChars using
# $chunkSize and $segSize to more quickly generate it.
sub randfile {
	my ($size,$filename) = @_;
	if (-f $filename) {
		return 0;
	}
	if ($size < 1) {
		return 0;
	}
	my $currSize = 0;
	my $chunkSize = $dataChunkSize / $subChunkDivider;
	my $segSize = $chunkSize / $segmentDivider;

	# else...
	my $segsPerChunk = int($chunkSize / $segSize);
	my $i;
	my $dataSize = length($randomChars);
	my $start;

	my $count = 0;
	if (open(FILE,">$filename")) {
		while ($currSize < $size) {
			my $chunk = "";
			for ($i = 0; $i < $segsPerChunk; $i++) {
				$start = int(rand($dataSize));
				$chunk .= substr($randomChars,$start,$segSize);
			}
			if (length($chunk) > ($size-$currSize)) {
				$chunk = substr($chunk,0,$size-$currSize);
			}
			syswrite(FILE,$chunk);
			$currSize += length($chunk);
			$count++;
		}
		close(FILE);
		print "Wrote data file in $count chunks.\n";
		return 1;
	} else {
		return 0;
	}
}

# pre_generate($size) pre-generates an array containing random chars
# to a maximum of $size bytes. The array is assumed to be @randomChars
# because we don't want to be passing say, a 500MB an array backwards
# and forwards
sub pre_generate {
	my $size = $_[0];
	my $count;
	my $max = 255;
	my $number;
	my $lastPercent = -1;
	my $segSize = $dataChunkSize / 10 / 50;
	$size += ($segSize*2);	# Give ourselves a bit of buffer.
	for ($count = 1; $count <= $size; $count++) {
		$randomChars .= chr(int(rand($max)));
		$number = int($count / $size * 100);
		if ($number % 10 == 0 && $lastPercent ne $number) {
			print "\t\t$number% of random data chunk generated.\n";
			$lastPercent = $number;
		}
	}
}

##############################################################################
# Subroutines
##############################################################################

if (getopts('hc:f:s:C:S:',\%opts)) {
	usage() if (defined($opts{h}));
	if (defined($opts{c})) {
		if ($opts{c} =~ /^\d+$/) {
			$dataChunkSize = $opts{c} * 1024 * 1024;
		}
	}
	if (defined($opts{f})) {
		$filename = $opts{f};
	} else {
		usage();
	}

	if (defined($opts{s}) && $opts{s} =~ /^\d+$/) {
		$size = $opts{s} * 1024 * 1024;
		$sizeMB = $opts{s};
	}
	if (defined($opts{C})) {
		if ($opts{C} =~ /^\d+$/ && $opts{C} != 0) {
			$subChunkDivider = $opts{C};
		} else {
			usage("Sub-chunk divider must be a positive integer.");
		}
	}
	if (defined($opts{S})) {
		if ($opts{S} =~ /^\d+$/ && $opts{S} != 0) {
			$segmentDivider = $opts{S};
		} else {
			usage("Segment divider must be a positive integer.");
		}
	}
}

print "Progress:\n";
print "\tPre-generating random data chunk. (This may take a while.)\n";
pre_generate($dataChunkSize);

print "\tCreating $sizeMB MB file $filename\n";
randfile($size,$filename);
